"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DevPacker = void 0;
const events_1 = require("events");
const fs_1 = require("fs");
const path_1 = require("path");
const stream_1 = require("stream");
const packer_utils_1 = require("@riejs/packer-utils");
const clone_deep_1 = __importDefault(require("clone-deep"));
const http_proxy_1 = require("http-proxy");
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const koa_1 = __importDefault(require("koa"));
const koa_static_1 = __importDefault(require("koa-static"));
const webpack_1 = __importDefault(require("webpack"));
const middleware_hot_1 = require("./middleware-hot");
const webpack_base_1 = require("../conf/webpack.base");
const webpack_client_1 = require("../conf/webpack.client");
const webpack_server_1 = require("../conf/webpack.server");
const file_1 = require("../utlis/file");
const merge_1 = require("../utlis/merge");
const template_1 = require("../utlis/template");
const MIN_PORT = 50000;
const MAX_PORT = 50999;
const buildingHtml = fs_1.readFileSync(path_1.resolve(__dirname, '../../building.html'), 'utf-8');
const defalutTemplate = fs_1.readFileSync(path_1.resolve(__dirname, '../../template.html'), 'utf-8');
/**
 * 发送EventSource到客户端
 */
const sendEventSource = (event, data) => `event:${event}\ndata: ${data}\n\n`;
/**
 * 开发模式构建
 */
/* eslint-disable no-param-reassign */
class DevPacker extends events_1.EventEmitter {
    constructor({ route, dir, template, packerOption }) {
        super();
        this.serverBundle = path_1.resolve(__dirname, '../single-vue/server.json');
        this.route = '';
        this.streams = {};
        this.clientConfig = clone_deep_1.default(webpack_base_1.config);
        this.serverConfig = clone_deep_1.default(webpack_base_1.config);
        this.readyStatus = { client: false, server: false };
        this.dist = path_1.resolve(__dirname, '../../tmp');
        this.isCompiling = false;
        this.route = route;
        const serverDist = path_1.resolve(this.dist, `./server${this.route}`);
        const clientDist = path_1.resolve(this.dist, `./client${this.route}`);
        if (!fs_1.existsSync(serverDist)) {
            fs_1.mkdirSync(serverDist, { recursive: true });
        }
        if (!fs_1.existsSync(clientDist)) {
            fs_1.mkdirSync(clientDist, { recursive: true });
        }
        file_1.copy([
            { from: path_1.resolve(__dirname, '../single-vue/server/dev'), to: path_1.resolve(this.dist, './server') },
            { from: path_1.resolve(__dirname, '../single-vue/client/dev'), to: clientDist },
        ]);
        this.serverConfig = webpack_server_1.getConfig(this.serverConfig, {
            mode: 'development',
            entry: path_1.resolve(dir, './index.vue'),
            dist: serverDist,
            publicPath: this.route,
            onProgress: (progress) => {
                if (progress >= 1) {
                    this.ready('server');
                }
            },
        });
        this.serverConfig.plugins.push(new html_webpack_plugin_1.default({
            templateContent: template_1.getServerTemplate(template || defalutTemplate, { isDev: true }),
            inject: false,
            filename: path_1.resolve(serverDist, './template.html'),
        }));
        this.serverConfig = merge_1.merge(this.serverConfig, packerOption === null || packerOption === void 0 ? void 0 : packerOption.server);
        this.clientConfig = webpack_client_1.getConfig(this.clientConfig, {
            mode: 'development',
            entry: path_1.resolve(dir, './index.vue'),
            publicPath: `${this.route}/`,
            dist: clientDist,
            hmrPath: this.hmrPath,
            onProgress: (progress) => {
                if (progress >= 1) {
                    this.ready('client');
                }
                else {
                    this.sendToStreams({ percent: progress });
                }
            },
        });
        this.clientConfig = merge_1.merge(this.clientConfig, packerOption === null || packerOption === void 0 ? void 0 : packerOption.client);
    }
    get progress() {
        return `${this.route}/progress`;
    }
    get hmrPath() {
        return `${this.route}/__webpack_hmr`;
    }
    async getBuildingRenderer() {
        await this.initDevServer();
        return {
            renderToString: (ctx, callback) => {
                if (!this.isCompiling) {
                    this.startCompile();
                }
                callback(null, buildingHtml.replace('{progressRoute}', this.progress));
            },
        };
    }
    proxy(context) {
        this.proxyServer.web(context.req, context.res);
    }
    async initDevServer() {
        return new Promise(async (resolve, reject) => {
            const port = await packer_utils_1.getUsablePort(MIN_PORT, MAX_PORT);
            if (port < 0) {
                reject(new Error('@riejs/packer-vue2 cannot create dev server, please retry.'));
                return;
            }
            const app = new koa_1.default();
            app.listen(port, () => {
                this.devServer = app;
                this.proxyServer = http_proxy_1.createProxyServer({ target: `http://127.0.0.1:${port}` });
                resolve();
            });
        });
    }
    startCompile() {
        this.isCompiling = true;
        this.initServerCompiler(this.serverConfig);
        this.initClientCompiler(this.clientConfig);
        this.devServer.use(async (ctx, next) => {
            if (ctx.path === this.progress) {
                const stream = new stream_1.PassThrough();
                const id = `stream-${Date.now()}`;
                this.streams[id] = stream;
                ctx.req.on('close', () => this.removeStream(id));
                ctx.req.on('finish', () => this.removeStream(id));
                ctx.req.on('error', () => this.removeStream(id));
                stream.write(sendEventSource('start', ''));
                ctx.type = 'text/event-stream';
                ctx.body = stream;
            }
            else {
                await next();
            }
        });
        this.devServer.use(koa_static_1.default(path_1.resolve(this.dist, './client')));
    }
    initClientCompiler(config) {
        const clientCompiler = webpack_1.default(config);
        clientCompiler.watch({}, (err, stats) => {
            if (err) {
                console.log(err);
                throw err;
            }
            const stat = stats.toJson();
            stat.errors.forEach(err => console.error(err));
            stat.warnings.forEach(err => console.warn(err));
        });
        const hotMiddleware = middleware_hot_1.hot(clientCompiler, { heartbeat: 2000, path: this.hmrPath });
        this.devServer.use(hotMiddleware);
    }
    initServerCompiler(config) {
        webpack_1.default(config).watch({}, (err, stats) => {
            if (err) {
                console.log(err);
                throw err;
            }
            const stat = stats.toJson();
            stat.errors.forEach(err => console.error(err));
            stat.warnings.forEach(err => console.warn(err));
        });
    }
    sendToStreams(data) {
        const eventStr = sendEventSource('message', JSON.stringify(data));
        Object.keys(this.streams).forEach(streamId => { var _a; return (_a = this.streams[streamId]) === null || _a === void 0 ? void 0 : _a.write(eventStr); });
    }
    removeStream(id) {
        this.streams[id] = null;
    }
    removeAllStreams() {
        Object.keys(this.streams).forEach((streamId) => {
            if (this.streams[streamId]) {
                this.streams[streamId].end();
                this.removeStream(streamId);
            }
        });
    }
    ready(type) {
        this.readyStatus[type] = true;
        if (this.readyStatus.server && this.readyStatus.client) {
            this.readyStatus.server = false;
            this.readyStatus.client = false;
            this.emit('ready', { dist: this.dist });
            this.sendToStreams({ percent: 100, msg: 'done' });
            this.removeAllStreams();
        }
    }
}
exports.DevPacker = DevPacker;
